#!moo verb bless_for_entry --on $room

# pylint: disable=return-outside-function,undefined-variable

"""
This verb is called by an exit to allow an object special permission to enter a room. Two properties on the room store
thing and the task_id of the calling task. The idea behind blessed objects is to allow an object temporary permission
to enter. The permission is only granted if the request is made by an object that is an entrance into the room.

The idea here is that a normally inaccessible room can be entered providing you use an exit that leads into the room.
In addition, the task ID of the task that asked for the blessing is stored, so that there is no way an object can
become blessed, and then later gain entry to the room. The object being blessed is only allowed to enter once per
blessing. Once the object has moved into the room, it's blessed status is removed by resetting the blessed_object
property in the room to $nothing.
"""

from moo.core import api

thing = args[0]
this.set_property("blessed_object", thing)
this.set_property("blessed_task_id", api.task_id)



import pytest

from moo.core import code, lookup, create
from moo.core.models import Object


@pytest.mark.django_db(transaction=True, reset_sequences=True)
@pytest.mark.parametrize("t_init", ["default"], indirect=True)
def test_parse_keyexp(t_init: Object, t_wizard: Object):
    printed = []

    def _writer(msg):
        printed.append(msg)

    with code.context(t_wizard, _writer):
        system = lookup(1)
        lock_utils = system.get_property("lock_utils")
        assert lock_utils is not None

        # Example 1: simple object number
        keyexp = "#123"
        key = lock_utils.parse_keyexp(keyexp)
        assert key == 123

        # Example 3: two objects
        keyexp = "#12 || #34"
        key = lock_utils.parse_keyexp(keyexp)
        assert key == ["||", 12, 34]

        # Example 4: negation
        keyexp = "!#56"
        key = lock_utils.parse_keyexp(keyexp)
        assert key == ["!", 56]

        # Example from docs:
        keyexp = "#45 && ?#46 && (#47 || !#48)"
        key = lock_utils.parse_keyexp(keyexp)
        assert key == ["&&", ["&&", 45, ["?", 46]], ["||", 47, ["!", 48]]]

@pytest.mark.django_db(transaction=True, reset_sequences=True)
@pytest.mark.parametrize("t_init", ["default"], indirect=True)
def test_eval_key(t_init: Object, t_wizard: Object):
    printed = []

    def _writer(msg):
        printed.append(msg)

    with code.context(t_wizard, _writer):
        system = lookup(1)
        lock_utils = system.get_property("lock_utils")
        assert lock_utils is not None

        # direct object matches
        assert lock_utils.eval_key(15, lookup(15)) is True
        assert lock_utils.eval_key(["!", 16], lookup(16)) is False
        assert lock_utils.eval_key(["!", 16], lookup(15)) is False
        assert lock_utils.eval_key(["||", 3, 4], lookup(3)) is True
        assert lock_utils.eval_key(["||", 3, 4], lookup(5)) is False

        # test the "?" operator
        assert lock_utils.eval_key(["?", 10], lookup(10)) is False  # not present
        system.set_property("key", 10)
        system.save()
        assert lock_utils.eval_key(["?", 1], lookup(10)) is True

        player = lookup("Player")
        thing1 = create("thing1", parents=[system.thing], location=player)
        thing2 = create("thing2", parents=[system.thing], location=player)
        assert lock_utils.eval_key(["&&", thing1.id, thing2.id], t_wizard) is False
        assert lock_utils.eval_key(["&&", thing1.id, thing2.id], player) is True



# -*- coding: utf-8 -*-
from .base import *  # pylint: disable=wildcard-import

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "django-insecure-06xo%2f39ohlfrgu&)%1lsrlx6jqy)s%#_-$rc5l_z1+p6w(=r"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ["moo.dev.shacklyn.net", "probe.cluster.local"]
CSRF_TRUSTED_ORIGINS = ["https://moo.dev.shacklyn.net", "https://probe.cluster.local"]

# Database
# https://docs.djangoproject.com/en/3.1/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.postgresql",
        "NAME": "termiverse",
        "HOST": "termiverse-db.dev.shacklyn.net",
        "USER": "termiverse",
        "PASSWORD": "termiverse",
    }
}

# CACHES = {
#     'default': {
#         'BACKEND': 'redis_cache.RedisCache',
#         'LOCATION': 'redis:6379',
#     }
# }



# Generated by Django 5.0.2 on 2024-03-06 01:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0008_alter_relationship_child_alter_relationship_parent"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="access",
            unique_together={
                ("object", "verb", "property", "rule", "permission", "type", "accessor", "group", "weight")
            },
        ),
    ]



import json

import pytest

from ..models import Object, Property


@pytest.mark.django_db(transaction=True, reset_sequences=True)
def test_child_inherits_parent_property(t_init: Object):
    room_class = Object.objects.get(name="room class")
    parent_description = room_class.properties.get(name="description")
    room = Object.objects.create(name="new room")
    room.parents.add(room_class)
    description = room.get_property(name="description")
    assert description == json.loads(parent_description.value)


@pytest.mark.django_db(transaction=True, reset_sequences=True)
def test_child_owns_inherited_property(t_init: Object):
    player = Object.objects.get(name="Player")
    room_class = Object.objects.get(name="room class")
    room = Object.objects.create(name="new room", owner=player)
    room.parents.add(room_class)
    description = room.get_property(name="description", recurse=False, original=True)
    assert description.origin == room


@pytest.mark.django_db(transaction=True, reset_sequences=True)
def test_returned_property_is_from_correct_object(t_init: Object):
    player = Object.objects.get(name="Player")
    room_class = Object.objects.get(name="room class")
    room = Object.objects.create(name="new room", owner=player)
    room.parents.add(room_class)
    description = room.get_property(name="description", recurse=False, original=True)
    assert description.owner == player


@pytest.mark.django_db(transaction=True, reset_sequences=True)
def test_property_inheritance_can_change_after_save(t_init: Object, t_wizard: Object):
    player = Object.objects.get(name="Player")
    o = Object.objects.create(name="new object", owner=player)
    p = Object.objects.create(name="new parent", owner=t_wizard)
    p.set_property("test_post_creation", "There's not much to see here.", owner=t_wizard)
    o.parents.add(p)

    description = o.get_property(name="test_post_creation", recurse=False, original=True)
    assert description.owner == player

    # You normally don't want to change the inheritence after creation, but it should be possible
    description = p.get_property(name="test_post_creation", recurse=False, original=True)
    description.inherit_owner = True
    description.save()

    # you just need to delete the inherited property and re-add the parent to re-inherit it with the correct owner
    o.get_property(name="test_post_creation", recurse=False, original=True).delete()
    o.parents.remove(p)
    o.parents.add(p)

    description = o.get_property(name="test_post_creation", recurse=False, original=True)
    assert description.owner == t_wizard



# Generated by Django 4.2.7 on 2023-12-16 19:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0004_accessibleobject_accessibleproperty_accessibleverb_and_more"),
    ]

    operations = [
        migrations.AddConstraint(
            model_name="verbname",
            constraint=models.UniqueConstraint(models.F("verb"), models.F("name"), name="unique_verb_name"),
        ),
    ]



import pytest

from moo.core import api, code, parse, lookup
from moo.core.models import Object, Player


@pytest.mark.django_db(transaction=True, reset_sequences=True)
@pytest.mark.parametrize("t_init", ["default"], indirect=True)
def test_basic_dig_and_tunnel(t_init: Object, t_wizard: Object):
    printed = []

    def _writer(msg):
        printed.append(msg)

    t_player = lookup("Player")
    with code.context(t_wizard, _writer) as ctx:
        home_location = t_wizard.location
        parse.interpret(ctx, "@dig north to Another Room")
        another_room = lookup("Another Room")
        assert printed == [
            '[color yellow]Dug an exit north to "Another Room".[/color yellow]',
        ]
        printed.clear()

        parse.interpret(ctx, "@dig north to Another Room")
        assert printed == ["[color red]There is already an exit in that direction.[/color red]"]
        printed.clear()

    with code.context(t_player, _writer) as ctx:
        with pytest.warns(RuntimeWarning, match=r"ConnectionError") as warnings:
            parse.interpret(ctx, "go north")
        assert [str(x.message) for x in warnings.list] == [
            f"ConnectionError(#{t_player.pk} (Player)): You leave #{home_location.pk} (The Laboratory).",
            f"ConnectionError(#{t_wizard.pk} (Wizard)): #{t_player.pk} (Player) leaves #{home_location.pk} (The Laboratory).",
            f"ConnectionError(#{t_player.pk} (Player)): You arrive at #{another_room.pk} (Another Room)."
        ]
        t_player.refresh_from_db()
        assert t_player.location.name == "Another Room"
        printed.clear()

    with code.context(t_wizard, _writer) as ctx:
        with pytest.warns(RuntimeWarning, match=r"ConnectionError") as warnings:
            parse.interpret(ctx, "go north")
        assert [str(x.message) for x in warnings.list] == [
            f"ConnectionError(#{t_wizard.pk} (Wizard)): You leave #{home_location.pk} (The Laboratory).",
            f"ConnectionError(#{t_wizard.pk} (Wizard)): You arrive at #{another_room.pk} (Another Room).",
            f"ConnectionError(#{t_player.pk} (Player)): #{t_wizard.pk} (Wizard) arrives at #18 (Another Room)."
        ]
        api.caller.refresh_from_db()
        api.player.refresh_from_db()
        parse.interpret(ctx, f"@tunnel south to {home_location.name}")
        assert printed == [
            f'[color yellow]Tunnelled an exit south to "{home_location.name}".[/color yellow]',
        ]
        assert t_player.location.get_property('exits')
        printed.clear()

    with code.context(t_player, _writer) as ctx:
        with pytest.warns(RuntimeWarning, match=r"ConnectionError") as warnings:
            parse.interpret(ctx, "go south")
        assert [str(x.message) for x in warnings.list] == [
            f"ConnectionError(#{t_player.pk} (Player)): You leave #{another_room.pk} (Another Room).",
            f"ConnectionError(#{t_wizard.pk} (Wizard)): #{t_player.pk} (Player) leaves #{another_room.pk} (Another Room).",
            f"ConnectionError(#{t_player.pk} (Player)): You arrive at #{home_location.pk} (The Laboratory)."
        ]
        t_player.refresh_from_db()
        assert t_player.location.name == home_location.name
        printed.clear()

    with code.context(t_player, _writer) as ctx:
        parse.interpret(ctx, "@exits")
        assert printed == [
            "[color cyan]Exits defined for this room:[/color cyan]",
            f"- [color yellow]north from The Laboratory[/color yellow] (Aliases: north) to [color green]Another Room[/color green] (#{another_room.pk})"
        ]
        printed.clear()

    with code.context(t_player, _writer) as ctx:
        parse.interpret(ctx, "@entrances")
        assert printed == [
            "[color cyan]Entrances defined for this room:[/color cyan]",
            f"- [color yellow]south from Another Room[/color yellow] (Aliases: south) to [color green]The Laboratory[/color green] (#{home_location.pk})"
        ]
        printed.clear()



#!moo verb test-args --on "player class" --dspec this

# pylint: disable=return-outside-function,undefined-variable

from moo.core import api

if args is not None:  # pylint: disable=undefined-variable
    print(f"METHOD:{args}:{kwargs}")  # pylint: disable=undefined-variable



#!moo verb accept --on $root_class

# pylint: disable=return-outside-function,undefined-variable

"""
This verb is used to control what objects are permitted to be placed inside other objects. If this verb returns
`False`, then the object where cannot be moved into this object. Conversely, if the verb returns a non-zero value, the
object is allowed to be placed inside this object. The verb is called by the server when it changes the `location`
property of an object.

The `$root_class` definition returns `False`. In this case, no objects are allowed inside any objects that are
children of the `$root_class`.
"""

thing = args[0]

return False



import pytest

from .. import code, parse, create
from ..models import Object, Verb


@pytest.mark.django_db(transaction=True, reset_sequences=True)
def test_caller_can_invoke_trivial_verb(t_init: Object, t_wizard: Object):
    printed = []
    description = t_wizard.location.properties.get(name="description")

    def _writer(msg):
        printed.append(msg)

    with code.context(t_wizard, _writer):
        writer = code.context.get("writer")
        globals = code.get_default_globals()  # pylint: disable=redefined-builtin
        globals.update(code.get_restricted_environment("__main__", writer))
        src = "from moo.core import api\napi.caller.invoke_verb('inspect')"
        code.r_exec(src, {}, globals)
        assert printed == [description.value]


@pytest.mark.django_db(transaction=True, reset_sequences=True)
def test_override_verb_in_subclass(t_init: Object, t_wizard: Object):
    printed = []

    def _writer(msg):
        printed.append(msg)

    with code.context(t_wizard, _writer):
        root = create("Root Class")
        root.add_verb("accept", code="return False")
        room = create("Test Room", parents=[root], location=None)
        with pytest.raises(PermissionError):
            test_obj = create("Test Object", location=room)
        room.add_verb("accept", code="return True")
        test_obj = create("Test Object", location=room)
        assert test_obj.location == room


@pytest.mark.django_db(transaction=True, reset_sequences=True)
def test_args_is_null_when_using_parser(t_init: Object, t_wizard: Object):
    printed = []

    def _writer(msg):
        printed.append(msg)

    with code.context(t_wizard, _writer) as ctx:
        parse.interpret(ctx, "test-args-parser")
    assert printed == ["PARSER"]


@pytest.mark.django_db(transaction=True, reset_sequences=True)
def test_args_is_not_null_when_using_eval(t_init: Object, t_wizard: Object):
    printed = []

    def _writer(msg):
        printed.append(msg)

    verb = Verb.objects.get(names__name="test-args")
    with code.context(t_wizard, _writer):
        code.interpret(verb.code, "__main__")
    assert printed == ["METHOD:():{}"]


@pytest.mark.django_db(transaction=True, reset_sequences=True)
def test_args_when_calling_multiple_verbs(t_init: Object, t_wizard: Object):
    printed = []

    def _writer(msg):
        printed.append(msg)

    with code.context(t_wizard, _writer) as ctx:
        parse.interpret(ctx, "test-nested-verbs")
    assert printed == list(range(1, 11))


@pytest.mark.django_db(transaction=True, reset_sequences=True)
def test_write_to_caller(t_init: Object, t_wizard: Object):
    printed = []

    def _writer(msg):
        printed.append(msg)

    with code.context(t_wizard, _writer):
        with pytest.warns(RuntimeWarning, match=r"ConnectionError\(\#3 \(Wizard\)\)\: TEST_STRING"):
            code.interpret("from moo.core import api, write\nwrite(api.caller, 'TEST_STRING')", "__main__")

@pytest.mark.django_db(transaction=True, reset_sequences=True)
def test_parse_with_wildard(t_init: Object, t_wizard: Object):
    box = Object.objects.create(name="box")
    box.location = t_wizard.location
    box.save()
    printed = []

    def _writer(msg):
        printed.append(msg)

    with code.context(t_wizard, _writer):
        lex = parse.Lexer("desc box")
        parser = parse.Parser(lex, t_wizard)
        verb = parser.get_verb()
        assert verb.names.filter(name="describe").exists()

@pytest.mark.django_db(transaction=True, reset_sequences=True)
def test_verb_passthrough(t_init: Object, t_wizard: Object):
    superbox = create(name="superbox", owner=t_wizard)
    superbox.location = t_wizard.location
    superbox.save()

    box = create(name="box", parents=[superbox], owner=t_wizard)
    box.location = t_wizard.location
    box.save()

    printed = []

    def _writer(msg):
        printed.append(msg)

    with code.context(t_wizard, _writer):
        superbox.add_verb("testpassthrough", code="""return "Superbox verb." """)
        box.add_verb("testpassthrough", code="""return "%s with some extra stuff." % passthrough() """)
        result = box.invoke_verb("testpassthrough")
        assert result == "Superbox verb. with some extra stuff."



#!moo verb @add-entrance --on $room --dspec any

# pylint: disable=return-outside-function,undefined-variable

"""
This is a player command used to add an entrance to the current room. This follows much the same sequence as for the
`@add-exit`. An attempt is made to match the direct object supplied with an object in the room. If this fails, the verb
is aborted with a suitable error message.

Otherwise, if the object found is a descendant of the `$exit` class, then the exit is checked to make sure it goes to
this room. If this is the case, then the exit is added as an entrance using the room's `add_entrance` verb.
"""

from moo.core import api

door = api.parser.get_dobj()
if not door.is_a(_.exit):
    print("[color red]The specified object is not an exit.[/color red]")
    return

this.add_entrance(door)
print(f'[color yellow]Added entrance "{door.name}".[/color yellow]')



"""
A variety of error classes
"""


class UserError(Exception):
    """
    This is the superclass for all Errors that may be generated which
    should be reported to the user who "caused" them. At construction,
    additional information can be supplied that will be presented to the
    user if they are a "wizard" user.
    """

    def __init__(self, message, data=None):
        self._message = message
        self.data = data

    def __str__(self):
        return str(self._message)

    def __repr__(self):
        data = ""
        if self.data:
            data += "\n    " + str(self.data)
        return str(self._message) + data


class UsageError(UserError):
    """
    This exception is available as a convenience to verbs that
    wish to print an error message and exit (rolling back any changes).
    """


class AmbiguousObjectError(UserError):
    """
    When this class is raised, it means that at some point a single
    object was expected, but multiple ones were found instead.
    When printed to the user, this shows a list of matching items,
    along with their unique IDs, so the user can choose the correct one.
    """

    def __init__(self, name, matches, message=None):
        self.name = name
        if message:
            result = message
        else:
            result = 'When you say, "' + self.name + '", do you mean '
            matches = list(matches)
            for index in range(len(matches)):  # pylint: disable=consider-using-enumerate
                match = matches[index]
                if index < len(matches) - 1:
                    result += ", "
                elif index == len(matches) - 1:
                    result += " or "
                result += str(match)
            result += "?"
        UserError.__init__(self, result, matches)


class AmbiguousVerbError(UserError):
    """
    When this class is raised, it means that when searching for a verb,
    multiple possibilities were encountered. There are some uncertainties
    about whether this should be presented to the user or not, since there
    is little they can do about it, unless they are wizardly.
    When printed to the user, this shows a list of objects with matching verbs.
    """

    def __init__(self, name, matches):
        self.name = name
        result = 'More than one object defines "' + self.name + '": '
        matches = list(matches)
        for match in matches:
            index = matches.index(match)
            if index < len(matches) - 1:
                result += ", "
            elif index == len(matches) - 1:
                result += " and "
            result = result + str(match)
        result = result + "."
        UserError.__init__(self, result, matches)


class AccessError(PermissionError):
    """
    A more specific kind of PermissionError.
    """

    def __init__(self, accessor, access_str, subject):
        self.subject = subject
        self.accessor = accessor
        self.access_str = access_str

        PermissionError.__init__(self, "%s is not allowed to '%s' on %s" % (str(accessor), access_str, str(subject)))


class RecursiveError(UserError):
    """
    Raised if the user attempts to put a container inside one of its contents,
    or set the parent of an object to one of its children. This is unlikely
    to happen because of user error, and may not need to be presented to the
    user, but will be for the time.
    """


class QuotaError(UserError):
    """
    Raised if the user tries to create objects he does not have enough quota for.
    """


class NoSuchPrepositionError(UserError):
    """
    Raised by the parser when the programmer attempts to retreive the object
    for a preposition that was not used in the sentence.
    """

    def __init__(self, prep):
        UserError.__init__(self, "I don't understand you.", prep)


class ExecutionError(UserError):
    """
    Raised when user code causes some kind of exception.
    """

    def __init__(self, code, e):
        self.code = code
        self.e = e
        UserError.__init__(self, "An error occurred in user code: %s" % e)



#!moo verb test-nested-verbs --on "player class" --dspec none

# pylint: disable=return-outside-function,undefined-variable

from moo.core import api

print(1)
api.caller.invoke_verb("test-nested-verbs-method", 1)



#!moo verb test-async-verb --on "player class" --dspec this

# pylint: disable=return-outside-function,undefined-variable

from moo.core import api, invoke

counter = 1
if args and len(args):  # pylint: disable=undefined-variable  # type: ignore
    counter = args[0] + 1  # pylint: disable=undefined-variable  # type: ignore

if counter < 10:
    return counter  # pylint: disable=return-outside-function  # type: ignore



#!moo verb remove_entrance --on $room

# pylint: disable=return-outside-function,undefined-variable

"""
This verb performs the opposite function to the `add_entrance` verb. It removes `entrance` from the room's list of
entrances. If it is not possible to remove `entrance` from the room's entrance list (normally because the object
that invoked the verb does not have the required permission) then the verb returns `False`. Otherwise, a successful
addition returns `True`.
"""

entrance = args[0]

if this.has_property("entrances"):
    entrances = this.get_property("entrances")
else:
    entrances = []

entrances.remove(entrance)
this.set_property("entrances", entrances)

return True



# pylint: disable=imported-auth-user
from django.contrib.auth.models import User
from django.core.management.base import BaseCommand

from ...models import Object, Player


class Command(BaseCommand):
    help = "Attach a Django user to an avatar in the game."

    def add_arguments(self, parser):
        parser.add_argument(
            "username", type=str, help="Name of the user to enable MOO access for."
        )
        parser.add_argument(
            "avatar", type=str, help="Name of the Avatar to grant to the user."
        )
        parser.add_argument(
            "--wizard",
            action="store_true",
            help="Optionally set the user as a wizard (superuser) inside the game.",
        )

    def handle(
        self, username, avatar, wizard=False, **kwargs
    ):  # pylint: disable=arguments-differ
        avatar = Object.objects.get(name=avatar, unique_name=True)
        user = User.objects.get(username=username)
        if not getattr(user, "player", None):
            player = Player()
            player.save()
            user.player = player
        user.player.avatar = avatar
        user.player.wizard = wizard
        user.player.save()



#!moo verb tell --on $root_class

# pylint: disable=return-outside-function,undefined-variable

"""
This verb is used to send a message from one object to another. The `$root_class` definition of this verb tests to see
if the object is a player, and if it is, uses the `write()` primitive to print the argument list on the player's screen,
if they are connected. However, this verb can be overridden to allow arbitrary objects to pass messages between each
other, or to augment the way the message is handled.

One simple example is that of an object that listens to everything that happens in a room. Every verb that needs to
send text to players uses the `tell` verb. If an object has its own `tell` verb, it too will be able to act upon the
messages sent between objects in a room.

The `$player` class overrides this verb to filter messages in two different ways, as show below:

if (typeof(this.gaglist) != LIST || !(player in this.gaglist))
    if (player != this and this.paranoid == 1)
        pass("<", player.name, "(", player, ")> ", @args);
    else
        pass(@args);
    endif
endif

Firstly, if the message comes from a player that we don't want to listen to - the player has been gagged - then the
message is thrown away. Secondly, if the player is being paranoid, and the message is not from ourselves, it is
prefaced with the name of the originating object. The pass primitive is used to allow the :tell verb of the parent
class to send the message after it has been modified.
"""

from moo.core import api, write

if this.is_player():
    for arg in args:
        write(this, arg)



#!moo verb test-async-verb-callback --on "player class" --dspec this

# pylint: disable=return-outside-function,undefined-variable

print(args[0])  # pylint: disable=undefined-variable  # type: ignore



#!moo verb g*et take --on $player

# pylint: disable=return-outside-function,undefined-variable

"""
This verb is called in the situation when another object tries to get or take a player. Given that this is a fairly
nonsensical thing to do, it is not allowed. Suitable messages are sent to the object that invoked the action, and
the player object.
"""



#!moo verb @lock_for_open lock_for_open --on $container --dspec this --ispec with:any

# pylint: disable=return-outside-function,undefined-variable

"""
This verb will lock the container with `object`. The container can only be opened if the player is holding `object`, or
the `object` is the player trying to open the container. The container will remained locked until it is unlocked.
"""

from moo.core import api

object = args[0] if args else api.parser.get_pobj("with")

this.set_property("open_key", object)



#!moo verb match_object --on $room

# pylint: disable=return-outside-function,undefined-variable

"""
This is the verb used to search the player's locale for an object that has the name or pseudonym name. The verb
searches the room contents and the players contents (or possessions) for a match. If a match is found, then the object
is returned. If name matches more than one object, then AmbiguousObjectError is raised. If no match is found, then
Object.DoesNotExist is raised.

The verb `match_object` is the one to use to map names of objects to object references, when referring to objects that the
player is able to see in his current location. This includes objects that the player might be carrying, but does not
include objects that are contained in other objects.
"""

from moo.core import api, AmbiguousObjectError

name = args[0]

qs = this.find(name)
if not qs:
    qs = api.player.find(name)
if not qs:
    raise this.DoesNotExist(name)
elif len(qs) > 1:
    raise AmbiguousObjectError(name, matches=qs.all())

return qs[0]



import pytest

from moo.core.models.object import Object

from .. import code


@pytest.mark.django_db(transaction=True, reset_sequences=True)
def test_eval_simple_command(t_init: Object, t_wizard: Object):
    def _writer(msg):
        raise RuntimeError("print was called unexpectedly")

    with code.context(t_wizard, _writer):
        writer = code.context.get("writer")
        globals = code.get_default_globals()  # pylint: disable=redefined-builtin
        globals.update(code.get_restricted_environment("__main__", writer))
        result = code.do_eval("dir()", {}, globals)
        assert result == []


@pytest.mark.django_db(transaction=True, reset_sequences=True)
def test_trivial_printing(t_init: Object, t_wizard: Object):
    printed = []

    def _writer(msg):
        printed.append(msg)

    with code.context(t_wizard, _writer):
        writer = code.context.get("writer")
        globals = code.get_default_globals()  # pylint: disable=redefined-builtin
        globals.update(code.get_restricted_environment("__main__", writer))
        result = code.do_eval("print('test')", {}, globals)
        assert result is None
        assert printed == ["test"]


@pytest.mark.django_db(transaction=True, reset_sequences=True)
def test_printing_imported_caller(t_init: Object, t_wizard: Object):
    printed = []

    def _writer(msg):
        printed.append(msg)

    with code.context(t_wizard, _writer):
        writer = code.context.get("writer")
        globals = code.get_default_globals()  # pylint: disable=redefined-builtin
        globals.update(code.get_restricted_environment("__main__", writer))
        src = "from moo.core import api\nprint(api.caller)"
        code.r_exec(src, {}, globals)
        assert printed == [t_wizard]



#!moo verb say --on $room --dspec any

# pylint: disable=return-outside-function,undefined-variable

"""
This verb provides one of the basic ways in which players communicate. The action of the :say verb is very simple: it
`tells` the player what s/he has just said, and tells everyone else what the player said. The text spoken is passed to
all the objects in a room, not just the players, through the `tell` verbs on the objects in the room.

By overriding this verb, it is possible to provide all sorts of effects that work on everything said in the room. For
example, you could redirect messages to other rooms, or repeat messages to provide cavernous echoes.
"""

from moo.core import api, lookup

if api.parser.words:
    message = " ".join(api.parser.words[1:])
else:
    message = " ".join(args)

api.caller.tell("You: " + message)
for obj in this.contents.all():
    if obj != api.caller:
        obj = lookup(obj.id)
        obj.tell(api.caller.name + ": " + message)



import logging
import warnings
import pytest
from django.test import override_settings

from .. import code, tasks
from ..models import Object, Verb


@pytest.mark.django_db(transaction=True, reset_sequences=True)
def test_simple_async_verb(t_init: Object, t_wizard: Object, caplog: pytest.LogCaptureFixture):
    printed = []

    def _writer(msg):
        printed.append(msg)

    verb = Verb.objects.get(names__name="test-async-verbs")
    with caplog.at_level(logging.INFO, "moo.core.tasks.background"):
        with code.context(t_wizard, _writer):
            verb()
    assert printed == [1]
    counter = 1
    for line in caplog.text.split("\n"):
        # Celery just loves emitting this when using the in-memory test broker, so ignore it
        if line.endswith("Reverting to default 'localhost'"):
            continue
        if not line:
            continue
        if "succeeded in" in line:
            continue
        counter += 1
        assert line.endswith(str(counter))


@pytest.mark.django_db(transaction=True, reset_sequences=True)
def test_simple_async_verb_callback(t_init: Object, t_wizard: Object, caplog: pytest.LogCaptureFixture):
    verb = Verb.objects.get(names__name="test-async-verb")
    callback = Verb.objects.get(names__name="test-async-verb-callback")
    with warnings.catch_warnings():
        warnings.simplefilter("ignore", category=RuntimeWarning)
        with caplog.at_level(logging.INFO, "moo.core.tasks.background"):
            tasks.invoke_verb(caller_id=t_wizard.pk, verb_id=verb.pk, callback_verb_id=callback.pk, this_id=verb.origin.pk)
    counter = 0
    for line in caplog.text.split("\n"):
        # Celery just loves emitting this when using the in-memory test broker, so ignore it
        if line.endswith("Reverting to default 'localhost'"):
            continue
        if not line:
            continue
        if "succeeded in" in line:
            continue
        counter += 1
        assert line.endswith(str(counter))



#!moo verb say --on "player class" --dspec any

# pylint: disable=return-outside-function,undefined-variable

from moo.core import api, write

if not args and not api.parser.has_dobj_str():  # pylint: disable=undefined-variable  # type: ignore
    print("What do you want to say?")
    return  # pylint: disable=return-outside-function  # type: ignore

if api.parser and api.parser.has_dobj_str():
    msg = api.parser.get_dobj_str()
else:
    msg = args[0]  # pylint: disable=undefined-variable  # type: ignore

for obj in api.caller.location.contents.all():
    write(obj, f"[bright_yellow]{api.caller.name}[/bright_yellow]: {msg}")



import pytest

from moo.core import code, parse, lookup
from moo.core.models import Object

@pytest.mark.django_db(transaction=True, reset_sequences=True)
@pytest.mark.parametrize("t_init", ["default"], indirect=True)
def test_say(t_init: Object, t_wizard: Object):
    printed = []

    def _writer(msg):
        printed.append(msg)

    with code.context(t_wizard, _writer) as ctx:
        with pytest.warns(RuntimeWarning, match=r"ConnectionError") as warnings:
            parse.interpret(ctx, "say Hello, world!")
        player = lookup("Player")
        assert [str(x.message) for x in warnings.list] == [
            f"ConnectionError(#{t_wizard.pk} (Wizard)): You: Hello, world!",
            f"ConnectionError(#{player.pk} (Player)): Wizard: Hello, world!"
        ]



#!moo verb describe --on $room --dspec this

# pylint: disable=return-outside-function,undefined-variable

from moo.core import lookup, api

obj = this

response = f"[bright_yellow]{obj.name}[/bright_yellow]\n"
if obj.has_property("description"):
    response += obj.get_property('description')
else:
    response += "[deep_sky_blue1]No description available.[/deep_sky_blue1]"

contents = obj.contents.filter(obvious=True)
if contents:
    response += "\n[yellow]Obvious contents:[/yellow]\n"
    for content in contents:
        response += f"{content.name}\n"

exits = obj.get_property("exits")
if exits:
    response += "\n[yellow]Exits:[/yellow]\n"
    for direction, _ in exits.items():
        response += f"{direction}\n"

return response



# -*- coding: utf-8 -*-
"""
Celery workers run the verb tasks.
"""

from celery import Celery
from kombu.serialization import register

from .core import moojson

app = Celery("moo")
app.config_from_object("django.conf:settings", namespace="CELERY")
app.autodiscover_tasks()

register("moojson", moojson.dumps, moojson.loads, content_type="application/x-moojson", content_encoding="utf-8")



#!moo verb announce --on $room

# pylint: disable=return-outside-function,undefined-variable

"""
This verb is a general purpose verb used to send a message to every object in the room except the player that invoked
it. This is intended to be the way other verbs pass messages to objects in a room. For example, when an exit is
activated, it uses `announce` to inform the other players in the room of what has happened.
"""

from moo.core import api

if api.parser.words:
    message = " ".join(api.parser.words[1:])
else:
    message = " ".join(args)

for obj in this.contents.all():
    if obj != api.caller:
        obj.tell(" ".join(args))



#!moo verb moveto --on $root_class

# pylint: disable=return-outside-function,undefined-variable

"""
This verb is used to change the location of an object to be `where`. This verb is intended to be used by any other verbs
that must move an object to another location.

One important point to note is that this uses the set_task_perms() primitive to set the task permissions to those of
the thing that is being moved.

Again, by overriding the verb definition on an object, it is possible to augment or change the way an object is moved.
For example, you could keep a list of places visited by simply recording the where objects in a list every time this
function is called.
"""

from moo.core import set_task_perms

where = args[0]

set_task_perms(this.owner)
this.location = where
this.save()



#!moo verb describe --on $root_class

# pylint: disable=return-outside-function,undefined-variable

"""
The `describe` verb is used to set the description property of an object.

This is only allowed if we have permission, determined using the $perm_utils:controls() verb.
By overriding this verb and the `description` verb, it is possible to completely change the
representation of an object description. This is done invisibly to anyone outside the object,
as long as you adhere to the same interface to `description` and `describe`.
"""

from moo.core import api

this.set_property("description", args[0])



# pylint: disable=undefined-variable

obj = args[0]

obj.allow("wizards", "anything")
obj.allow("owners", "anything")
obj.allow("everyone", "read")

if obj.kind == "verb":
    obj.allow("everyone", "execute")
elif obj.kind == "property":
    obj.allow("everyone", "read")
elif obj.kind == "object":
    obj.allow("everyone", "read")



import pytest

from moo.core import code, lookup, parse
from moo.core.models import Object, Verb


@pytest.mark.django_db(transaction=True, reset_sequences=True)
@pytest.mark.parametrize("t_init", ["default"], indirect=True)
def test_creation(t_init: Object, t_wizard: Object):
    printed = []

    def _writer(msg):
        printed.append(msg)

    with code.context(t_wizard, _writer) as ctx:
        parse.interpret(ctx, "make a widget")
        widget = lookup("widget")
        assert widget.location == t_wizard.location
        assert printed == [
            f"[color yellow]Created #{widget.id} (widget)[/color yellow]",
        ]


@pytest.mark.django_db(transaction=True, reset_sequences=True)
@pytest.mark.parametrize("t_init", ["default"], indirect=True)
def test_transmutation(t_init: Object, t_wizard: Object):
    printed = []

    def _writer(msg):
        printed.append(msg)

    with code.context(t_wizard, _writer) as ctx:
        parse.interpret(ctx, "make a jar from Generic Container")
        jar = lookup("jar")
        container = lookup("Generic Container")
        assert printed == [
            f"[color yellow]Created #{jar.id} (jar)[/color yellow]",
            f"[color yellow]Transmuted #{jar.id} (jar) to #{container.id} (Generic Container)[/color yellow]",
        ]


@pytest.mark.django_db(transaction=True, reset_sequences=True)
@pytest.mark.parametrize("t_init", ["default"], indirect=True)
def test_description(t_init: Object, t_wizard: Object):
    printed = []

    def _writer(msg):
        printed.append(msg)

    with code.context(t_wizard, _writer) as ctx:
        parse.interpret(ctx, "make a thingy from Root Class")
        thingy = lookup("thingy")
        root = lookup("Root Class")
        assert list(thingy.parents.all()) == [root]
        assert printed == [
            f"[color yellow]Created #{thingy.id} (thingy)[/color yellow]",
            f"[color yellow]Transmuted #{thingy.id} (thingy) to #4 (Root Class)[/color yellow]",
        ]
        printed.clear()

        # parse.interpret(ctx, "@describe thingy")
        # assert printed == [
        #     "[red]What do you want to describe that as?[/red]",
        # ]
        # printed.clear()

        parse.interpret(ctx, "@describe thingy as 'a dusty old widget'")
        parse.interpret(ctx, "look at thingy")
        print(printed)
        assert printed == [
            f"[color yellow]Description set for #{thingy.id} (thingy)[/color yellow]",
            "[bright_yellow]thingy[/bright_yellow]\n[deep_sky_blue1]a dusty old widget[/deep_sky_blue1]",
        ]



import pytest

from moo.core import api, code, create, lookup, parse
from moo.core.models import Object


def setup_doors(t_wizard: Object):
    rooms = lookup("Generic Room")
    room = create("Test Room", parents=[rooms])
    doors = lookup("Generic Exit")
    door = create("wooden door", parents=[doors], location=room)
    t_wizard.location = room
    t_wizard.save()
    api.caller.refresh_from_db()
    return room, door


@pytest.mark.django_db(transaction=True, reset_sequences=True)
@pytest.mark.parametrize("t_init", ["default"], indirect=True)
def test_creation(t_init: Object, t_wizard: Object):
    printed = []

    def _writer(msg):
        printed.append(msg)

    with code.context(t_wizard, _writer) as ctx:
        room, door = setup_doors(t_wizard)
        parse.interpret(ctx, "@dig north to Another Room through wooden door")
        assert printed == ['[color yellow]Dug an exit north to "Another Room".[/color yellow]']
        assert t_wizard.location == room
        assert room.has_property("exits")
        assert door in room.exits
        another_room = door.dest

        printed.clear()
        with pytest.warns(RuntimeWarning, match=r"ConnectionError") as warnings:
            parse.interpret(ctx, "go north")
        assert [str(x.message) for x in warnings.list] == [
            f"ConnectionError(#{t_wizard.pk} (Wizard)): You leave #{room.pk} (Test Room).",
            f"ConnectionError(#{t_wizard.pk} (Wizard)): You arrive at #{another_room.pk} (Another Room)."
        ]
        api.caller.refresh_from_db()
        assert not printed


@pytest.mark.django_db(transaction=True, reset_sequences=True)
@pytest.mark.parametrize("t_init", ["default"], indirect=True)
def test_locking(t_init: Object, t_wizard: Object):
    printed = []

    def _writer(msg):
        printed.append(msg)

    with code.context(t_wizard, _writer) as ctx:
        _, door = setup_doors(t_wizard)
        parse.interpret(ctx, "@dig north to Another Room through wooden door")
        assert printed == ['[color yellow]Dug an exit north to "Another Room".[/color yellow]']
        printed.clear()
        parse.interpret(ctx, "lock wooden door")
        assert printed == ["The door is locked."]
        assert door.is_locked()
        printed.clear()
        parse.interpret(ctx, "unlock wooden door")
        assert printed == ["The door is unlocked."]
        assert not door.is_locked()


@pytest.mark.django_db(transaction=True, reset_sequences=True)
@pytest.mark.parametrize("t_init", ["default"], indirect=True)
def test_open(t_init: Object, t_wizard: Object):
    printed = []

    def _writer(msg):
        printed.append(msg)

    with code.context(t_wizard, _writer) as ctx:
        _, door = setup_doors(t_wizard)
        parse.interpret(ctx, "@dig north to Another Room through wooden door")
        assert printed == ['[color yellow]Dug an exit north to "Another Room".[/color yellow]']
        printed.clear()
        parse.interpret(ctx, "open wooden door")
        assert printed == ["The door is open."]
        assert door.is_open.invoked_name == "is_open"
        assert door.is_open()
        printed.clear()
        parse.interpret(ctx, "look through wooden door")
        assert printed == ["[bright_yellow]Another Room[/bright_yellow]\n[deep_sky_blue1]There's not much to see here.[/deep_sky_blue1]"]
        printed.clear()
        parse.interpret(ctx, "close wooden door")
        assert printed == ["The door is closed."]
        assert not door.is_open()



# Generated by Django 5.1.4 on 2025-05-11 22:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0019_alter_prepositionspecifier_specifier'),
    ]

    operations = [
        migrations.AlterField(
            model_name='verb',
            name='direct_object',
            field=models.CharField(choices=[('this', 'this'), ('any', 'any'), ('none', 'none')], db_default='none', db_index=True, max_length=255),
        ),
    ]



#!moo verb tell_contents --on $room

# pylint: disable=return-outside-function,undefined-variable

"""
This verb tells us what things are visible in the room. It goes through the contents list of the room, and if it is not
dark, prints the name of the object in a nicely formatted way. Three different formats are available depending on the
value of the ctype property of the room. These are best illustrated by example.

Consider the a room in the LambdaCore database. With a ctype value of three (the default), the tell_contents verb
produces the following output:

    You see a newspaper and a fruitbat zapper here.
    Wizard is here.

This format provides the separation of player objects from other objects, and provides the list in a way that fits in
with the idea of a virtual reality. It is easy to read, and looks natural.

If the ctype value is changed to 2, the following is printed:

    You see Wizard, a newspaper, and a fruitbat zapper here.

This format treats players and objects the same, and is useful if you wish to hide the fact that an object is not a
player, or vice versa.

With a ctype value of one, the following is seen:

    Wizard is here.
    You see a newspaper here.
    You see a fruitbat zapper here.

This format provides the advantage of having each item on a separate line, although it does mean that rooms with a
large number of objects in might have excessively long contents lists.

Finally, with a ctype of zero, the following is seen:

    Contents:
    Wizard
    a newspaper
    a fruitbat zapper

This is the sort of listing obtained in traditional TinyMU* games. It benefits from clarity, but is not as natural as
any of the other forms.

If a value of ctype is set that is outside of the range zero thru three, then no contents list is printed. This can be
useful if you want to handle the room contents listing as part of the description of the room. Also, if the dark
property of a room is set to a non-zero value, then no contents list is printed.

As usual, this verb can be overridden to provide special effects. For example, you could apply a filter so that certain
objects do not appear in the printed contents of a room, even if they appear in the contents list. This can be use to
hide objects, for example, as part of a puzzle, or to vary how objects are seen, for example if you are looking through
water at something.
"""

if this.contents.exists():
    print("Contents:")
    for item in this.contents.all():
        print("  " + item.title())
else:
    print("It is empty.")



#!moo verb make --on $programmer --dspec any --ispec from:any

# pylint: disable=return-outside-function,undefined-variable

from moo.core import api, create, lookup

if not (api.parser.has_dobj_str()):
    print("[color yellow]What do you want to make?[/color yellow]")
    return

name = api.parser.get_dobj_str()
new_obj = create(name)
print("[color yellow]Created %s[/color yellow]" % new_obj)

if api.parser.has_pobj_str("from"):
    parent_name = api.parser.get_pobj_str("from")
    try:
        parent = lookup(parent_name)
        new_obj.parents.add(parent)
    except new_obj.DoesNotExist:
        print(f"[color red]No such object: {parent_name}[/color red]")
        return
    print("[color yellow]Transmuted %s to %s[/color yellow]" % (new_obj, parent))
